@@ -1,8 +1,6 @@
 """
 Utility functions to return a formatted name and description for a given view.
 """
-from __future__ import unicode_literals
-
 import re
 
 from django.utils.encoding import force_text
@@ -32,14 +30,18 @@ def dedent(content):
     unindented text on the initial line.
     """
     content = force_text(content)
-    whitespace_counts = [len(line) - len(line.lstrip(' '))
-                         for line in content.splitlines()[1:] if line.lstrip()]
+    lines = [line for line in content.splitlines()[1:] if line.lstrip()]
 
     # unindent the content if needed
-    if whitespace_counts:
-        whitespace_pattern = '^' + (' ' * min(whitespace_counts))
-        content = re.sub(re.compile(whitespace_pattern, re.MULTILINE), '', content)
-
+    if lines:
+        whitespace_counts = min([len(line) - len(line.lstrip(' ')) for line in lines])
+        tab_counts = min([len(line) - len(line.lstrip('\t')) for line in lines])
+        if whitespace_counts:
+            whitespace_pattern = '^' + (' ' * whitespace_counts)
+            content = re.sub(re.compile(whitespace_pattern, re.MULTILINE), '', content)
+        elif tab_counts:
+            whitespace_pattern = '^' + ('\t' * tab_counts)
+            content = re.sub(re.compile(whitespace_pattern, re.MULTILINE), '', content)
     return content.strip()
 
 
@@ -48,8 +50,8 @@ def camelcase_to_spaces(content):
     Translate 'CamelCaseNames' to 'Camel Case Names'.
     Used when generating names from view classes.
     """
-    camelcase_boundry = '(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))'
-    content = re.sub(camelcase_boundry, ' \\1', content).strip()
+    camelcase_boundary = '(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))'
+    content = re.sub(camelcase_boundary, ' \\1', content).strip()
     return ' '.join(content.split('_')).title()
 
 
@@ -63,3 +65,29 @@ def markup_description(description):
         description = escape(description).replace('\n', '<br />')
         description = '<p>' + description + '</p>'
     return mark_safe(description)
+
+
+class lazy_format:
+    """
+    Delay formatting until it's actually needed.
+
+    Useful when the format string or one of the arguments is lazy.
+
+    Not using Django's lazy because it is too slow.
+    """
+    __slots__ = ('format_string', 'args', 'kwargs', 'result')
+
+    def __init__(self, format_string, *args, **kwargs):
+        self.result = None
+        self.format_string = format_string
+        self.args = args
+        self.kwargs = kwargs
+
+    def __str__(self):
+        if self.result is None:
+            self.result = self.format_string.format(*self.args, **self.kwargs)
+            self.format_string, self.args, self.kwargs = None, None, None
+        return self.result
+
+    def __mod__(self, value):
+        return str(self) % value
