@@ -1,6 +1,4 @@
-#! /usr/bin/env python
-from __future__ import print_function
-
+#! /usr/bin/env python3
 import subprocess
 import sys
 
@@ -13,7 +11,7 @@ PYTEST_ARGS = {
 
 FLAKE8_ARGS = ['rest_framework', 'tests']
 
-ISORT_ARGS = ['--recursive', '--check-only', '--diff', '-o' 'uritemplate', '-p', 'tests', 'rest_framework', 'tests']
+ISORT_ARGS = ['--recursive', '--check-only', '--diff', 'rest_framework', 'tests']
 
 
 def exit_on_failure(ret, message=None):
