@@ -56,6 +56,8 @@ def pytest_configure(config):
             'django.contrib.staticfiles',
             'rest_framework',
             'rest_framework.authtoken',
+            'tests.authentication',
+            'tests.generic_relations',
             'tests.importable',
             'tests',
         ),
